@@ -1,11 +1,11 @@
 ;;; diff-hl.el --- Highlight uncommitted changes using VC -*- lexical-binding: t -*-
 
-;; Copyright (C) 2012-2023  Free Software Foundation, Inc.
+;; Copyright (C) 2012-2024  Free Software Foundation, Inc.
 
-;; Author:   Dmitry Gutov <dgutov@yandex.ru>
+;; Author:   Dmitry Gutov <dmitry@gutov.dev>
 ;; URL:      https://github.com/dgutov/diff-hl
 ;; Keywords: vc, diff
-;; Version:  1.9.2
+;; Version:  1.10.0
 ;; Package-Requires: ((cl-lib "0.2") (emacs "25.1"))
 
 ;; This file is part of GNU Emacs.
@@ -194,6 +194,25 @@ the NEW revision is not specified (meaning, the diff is against
 the current version of the file)."
   :type 'boolean)
 
+(defcustom diff-hl-update-async nil
+  "When non-nil, `diff-hl-update' will run asynchronously.
+
+This can help prevent Emacs from freezing, especially by a slow version
+control (VC) backend. It's disabled in remote buffers, though, since it
+didn't work reliably in such during testing."
+  :type 'boolean)
+
+;; Threads are not reliable with remote files, yet.
+(defcustom diff-hl-async-inhibit-functions (list #'diff-hl-with-editor-p
+                                                 #'file-remote-p)
+  "Functions to call to check whether asychronous method should be disabled.
+
+When `diff-hl-update-async' is non-nil, these functions are called in turn
+and passed the value `default-directory'.
+
+If any returns non-nil, `diff-hl-update' will run synchronously anyway."
+  :type '(repeat :tag "Predicate" function))
+
 (defvar diff-hl-reference-revision nil
   "Revision to diff against.  nil means the most recent one.")
 
@@ -290,7 +309,7 @@ the current version of the file)."
                      (listp vc-git-diff-switches)
                      (cl-remove-if-not
                       (lambda (arg)
-                        (member arg '("--histogram" "--patience" "--minimal")))
+                        (member arg '("--histogram" "--patience" "--minimal" "--textconv")))
                       vc-git-diff-switches))))
          (vc-hg-diff-switches nil)
          (vc-svn-diff-switches nil)
@@ -384,6 +403,28 @@ the current version of the file)."
       (nreverse res))))
 
 (defun diff-hl-update ()
+  "Updates the diff-hl overlay."
+  (if (and diff-hl-update-async
+           (not
+            (run-hook-with-args-until-success 'diff-hl-async-inhibit-functions
+                                              default-directory)))
+      ;; TODO: debounce if a thread is already running.
+      (make-thread 'diff-hl--update-safe "diff-hl--update-safe")
+    (diff-hl--update)))
+
+(defun diff-hl-with-editor-p (_dir)
+  (bound-and-true-p with-editor-mode))
+
+(defun diff-hl--update-safe ()
+  "Updates the diff-hl overlay. It handles and logs when an error is signaled."
+  (condition-case err
+      (diff-hl--update)
+    (error
+     (message "An error occurred in diff-hl--update: %S" err)
+     nil)))
+
+(defun diff-hl--update ()
+  "Updates the diff-hl overlay."
   (let ((changes (diff-hl-changes))
         (current-line 1))
     (diff-hl-remove-overlays)
@@ -465,8 +506,7 @@ the current version of the file)."
       (diff-hl-update))))
 
 (defun diff-hl-after-revert ()
-  (defvar revert-buffer-preserve-modes)
-  (when revert-buffer-preserve-modes
+  (when (bound-and-true-p revert-buffer-preserve-modes)
     (diff-hl-update)))
 
 (defun diff-hl-diff-goto-hunk-1 (historic)
@@ -541,7 +581,7 @@ in the source file, or the last line of the hunk above it."
             (let ((to-go (1+ (- line hunk-line))))
               (while (cl-plusp to-go)
                 (forward-line 1)
-                (unless (looking-at "^-")
+                (unless (looking-at "^[-\\]")
                   (cl-decf to-go))))))))))
 
 (defface diff-hl-reverted-hunk-highlight
@@ -609,6 +649,7 @@ in the source file, or the last line of the hunk above it."
                     (unless (yes-or-no-p (format "Revert current hunk in %s? "
                                                  file))
                       (user-error "Revert canceled")))
+                (widen)
                 (let ((diff-advance-after-apply-hunk nil))
                   (save-window-excursion
                     (diff-apply-hunk t)))
@@ -628,7 +669,7 @@ Move point to the beginning of the delineated hunk and return
 its end position."
   (let (end-marker)
     (save-excursion
-      (while (looking-at "[-+]") (forward-line 1))
+      (while (looking-at "[-+\\]") (forward-line 1))
       (dotimes (_i max-context)
         (unless (looking-at "@\\|[-+]")
           (forward-line 1)))
@@ -637,13 +678,14 @@ its end position."
                   (looking-at "@"))
         (diff-split-hunk)))
     (unless (looking-at "[-+]") (forward-line -1))
-    (while (looking-at "[-+]") (forward-line -1))
+    (while (looking-at "[-+\\]") (forward-line -1))
     (dotimes (_i max-context)
       (unless (looking-at "@\\|[-+]")
         (forward-line -1)))
     (unless (looking-at "@")
       (forward-line 1)
-      (diff-split-hunk))
+      (diff-split-hunk)
+      (forward-line -1))
     end-marker))
 
 (defun diff-hl-revert-hunk ()
@@ -706,6 +748,21 @@ its end position."
     (unless (eq backend 'Git)
       (user-error "Only Git supports staging; this file is controlled by %s" backend))))
 
+(defun diff-hl-stage-diff (orig-buffer)
+  (let ((patchfile (make-temp-file "diff-hl-stage-patch"))
+        success)
+    (write-region (point-min) (point-max) patchfile
+                  nil 'silent)
+    (unwind-protect
+        (with-current-buffer orig-buffer
+          (with-output-to-string
+            (vc-git-command standard-output 0
+                            patchfile
+                            "apply" "--cached" )
+            (setq success t)))
+      (delete-file patchfile))
+    success))
+
 (defun diff-hl-stage-current-hunk ()
   "Stage the hunk at or near point.
 
@@ -717,7 +774,8 @@ Only supported with Git."
          (file buffer-file-name)
          (dest-buffer (get-buffer-create " *diff-hl-stage*"))
          (orig-buffer (current-buffer))
-         (file-base (shell-quote-argument (file-name-nondirectory file)))
+         ;; FIXME: If the file name has double quotes, these need to be quoted.
+         (file-base (file-name-nondirectory file))
          success)
     (with-current-buffer dest-buffer
       (let ((inhibit-read-only t))
@@ -738,21 +796,11 @@ Only supported with Git."
         (insert (format "diff a/%s b/%s\n" file-base file-base))
         (insert (format "--- a/%s\n" file-base))
         (insert (format "+++ b/%s\n" file-base)))
-      (let ((patchfile (make-temp-file "diff-hl-stage-patch")))
-        (write-region (point-min) (point-max) patchfile
-                      nil 'silent)
-        (unwind-protect
-            (with-current-buffer orig-buffer
-              (with-output-to-string
-                (vc-git-command standard-output 0
-                                patchfile
-                                "apply" "--cached" ))
-              (setq success t))
-          (delete-file patchfile))))
+      (setq success (diff-hl-stage-diff orig-buffer)))
     (when success
       (if diff-hl-show-staged-changes
           (message (concat "Hunk staged; customize `diff-hl-show-staged-changes'"
-                           " to highlight only unstages changes"))
+                           " to highlight only unstaged changes"))
         (message "Hunk staged"))
       (unless diff-hl-show-staged-changes
         (diff-hl-update)))))
@@ -770,6 +818,85 @@ Only supported with Git."
   (unless diff-hl-show-staged-changes
     (diff-hl-update)))
 
+(defun diff-hl-stage-dwim (&optional with-edit)
+  "Stage the current hunk or choose the hunks to stage.
+When called with the prefix argument, invokes `diff-hl-stage-some'."
+  (interactive "P")
+  (if (or with-edit (region-active-p))
+      (call-interactively #'diff-hl-stage-some)
+    (call-interactively #'diff-hl-stage-current-hunk)))
+
+(defvar diff-hl-stage--orig nil)
+
+(define-derived-mode diff-hl-stage-diff-mode diff-mode "Stage Diff"
+  "Major mode for editing a diff buffer before staging.
+
+\\[diff-hl-stage-commit]"
+  (setq revert-buffer-function #'ignore))
+
+(define-key diff-hl-stage-diff-mode-map (kbd "C-c C-c") #'diff-hl-stage-finish)
+
+(defun diff-hl-stage-some (&optional beg end)
+  "Stage some or all of the current changes, interactively.
+Pops up a diff buffer that can be edited to choose the changes to stage."
+  (interactive "r")
+  (diff-hl--ensure-staging-supported)
+  (let* ((line-beg (and beg (line-number-at-pos beg t)))
+         (line-end (and end (line-number-at-pos end t)))
+         (file buffer-file-name)
+         (dest-buffer (get-buffer-create "*diff-hl-stage-some*"))
+         (orig-buffer (current-buffer))
+         ;; FIXME: If the file name has double quotes, these need to be quoted.
+         (file-base (file-name-nondirectory file)))
+    (with-current-buffer dest-buffer
+      (let ((inhibit-read-only t))
+        (erase-buffer)))
+    (diff-hl-diff-buffer-with-reference file dest-buffer nil 3)
+    (with-current-buffer dest-buffer
+      (let ((inhibit-read-only t))
+        (when end
+          (with-no-warnings
+            (let (diff-auto-refine-mode)
+              (diff-hl-diff-skip-to line-end)
+              (diff-hl-split-away-changes 3)
+              (diff-end-of-hunk)))
+          (delete-region (point) (point-max)))
+        (if beg
+            (with-no-warnings
+              (let (diff-auto-refine-mode)
+                (diff-hl-diff-skip-to line-beg)
+                (diff-hl-split-away-changes 3)
+                (diff-beginning-of-hunk)))
+          (goto-char (point-min))
+          (forward-line 3))
+        (delete-region (point-min) (point))
+        ;; diff-no-select creates a very ugly header; Git rejects it
+        (insert (format "diff a/%s b/%s\n" file-base file-base))
+        (insert (format "--- a/%s\n" file-base))
+        (insert (format "+++ b/%s\n" file-base)))
+      (let ((diff-default-read-only t))
+        (diff-hl-stage-diff-mode))
+      (setq-local diff-hl-stage--orig orig-buffer))
+    (pop-to-buffer dest-buffer)
+    (message "Press %s and %s to navigate, %s to split, %s to kill hunk, %s to undo, and %s to stage the diff after editing"
+             (substitute-command-keys "\\`n'")
+             (substitute-command-keys "\\`p'")
+             (substitute-command-keys "\\[diff-split-hunk]")
+             (substitute-command-keys "\\[diff-hunk-kill]")
+             (substitute-command-keys "\\[diff-undo]")
+             (substitute-command-keys "\\[diff-hl-stage-finish]"))))
+
+(defun diff-hl-stage-finish ()
+  (interactive)
+  (let ((count 0))
+    (when (diff-hl-stage-diff diff-hl-stage--orig)
+      (save-excursion
+        (goto-char (point-min))
+        (while (re-search-forward diff-hunk-header-re-unified nil t)
+          (cl-incf count)))
+      (message "Staged %d hunks" count)
+      (bury-buffer))))
+
 (defvar diff-hl-command-map
   (let ((map (make-sparse-keymap)))
     (define-key map "n" 'diff-hl-revert-hunk)
@@ -778,7 +905,7 @@ Only supported with Git."
     (define-key map "*" 'diff-hl-show-hunk)
     (define-key map "{" 'diff-hl-show-hunk-previous)
     (define-key map "}" 'diff-hl-show-hunk-next)
-    (define-key map "S" 'diff-hl-stage-current-hunk)
+    (define-key map "S" 'diff-hl-stage-dwim)
     map))
 (fset 'diff-hl-command-map diff-hl-command-map)
 
@@ -824,7 +951,7 @@ The value of this variable is a mode line template as in
     (remove-hook 'after-save-hook 'diff-hl-update t)
     (remove-hook 'after-change-functions 'diff-hl-edit t)
     (remove-hook 'find-file-hook 'diff-hl-update t)
-    (remove-hook 'after-revert-hook 'diff-hl-update t)
+    (remove-hook 'after-revert-hook 'diff-hl-after-revert t)
     (remove-hook 'magit-revert-buffer-hook 'diff-hl-update t)
     (remove-hook 'magit-not-reverted-hook 'diff-hl-update t)
     (remove-hook 'text-scale-mode-hook 'diff-hl-maybe-redefine-bitmaps t)
@@ -985,7 +1112,7 @@ CONTEXT-LINES is the size of the unified diff context, defaults to 0."
     (let* ((dest-buffer (or dest-buffer "*diff-hl-diff-buffer-with-reference*"))
            (backend (or backend (vc-backend file)))
            (temporary-file-directory
-            (if (file-directory-p "/dev/shm/")
+            (if (and (eq system-type 'gnu/linux) (file-directory-p "/dev/shm/"))
                 "/dev/shm/"
               temporary-file-directory))
            (rev
@@ -1019,7 +1146,7 @@ CONTEXT-LINES is the size of the unified diff context, defaults to 0."
 
 (defun diff-hl-git-index-revision (file object-name)
   (let ((filename (diff-hl-make-temp-file-name file
-                                               (concat ":" object-name)
+                                               (concat ";" object-name)
                                                'manual))
         (filebuf (get-file-buffer file)))
     (unless (file-exists-p filename)
